@@ -1,3 +1,4 @@
+from django.conf import settings
 from django.forms import Form, ModelForm
 from django.http import HttpResponse, HttpResponseRedirect
 from django.shortcuts import render_to_response
@@ -18,7 +19,7 @@ class DynamicResponse(object):
     """
     
     def __init__(self, context={}, **kwargs):
-
+        
         self.context = context
         self.status = context.get('status', CR_OK)
         for arg in kwargs:
@@ -28,13 +29,43 @@ class DynamicResponse(object):
         """
         Serializes the context as JSON, or returns a HTTP response with corresponding status.
         """
-
-        key, status_code = self.status
-        if status_code == 200:
-            return JsonResponse(self.context)
-        else:
-            return HttpResponse(status=status_code)
         
+        key, status_code = self.status
+        
+        if status_code == CR_OK[1]:
+            return JsonResponse(self.context)
+        
+        elif status_code == CR_INVALID_DATA[1]:
+            
+            # Output form errors when return status is invalid
+            if hasattr(self, 'extra') and getattr(settings, 'DYNAMICRESPONSE_JSON_FORM_ERRORS', False):
+                errors = {}
+                
+                for form in [value for key, value in self.extra.items() if isinstance(value, Form) or isinstance(value, ModelForm)]:
+                    if not form.is_valid():
+                        
+                        for key, val in form.errors.items():
+                            
+                            # Flatten field errors
+                            if isinstance(val, list):
+                                val = ' '.join(val)
+                            
+                            # Split general and field specific errors
+                            if key == '__all__':
+                                errors['general_errors'] = val
+                            else:
+                                
+                                # Field specific errors
+                                if not 'field_errors' in errors:
+                                    errors['field_errors'] = {}
+                                
+                                errors['field_errors'][key] = val
+                
+                return JsonResponse(errors, status=status_code)
+        
+        # Return blank response for all other status codes
+        return HttpResponse(status=status_code)
+    
     def full_context(self):
         """
         Returns context and extra context combined into a single dictionary.
@@ -51,47 +82,47 @@ class SerializeOrRender(DynamicResponse):
     For normal requests, the specified template is rendered.
     For API requests, the context is serialized and returned as JSON.
     """
-
+    
     def __init__(self, template, context={}, **kwargs):
         
         super(SerializeOrRender, self).__init__(context, **kwargs)
         self.template = template
-        
+    
     def render_response(self, request, response):
-            
+        
         if request.is_api:
             res = self.serialize()
         else:
             res = render_to_response(self.template, self.full_context(), RequestContext(request))
-
+        
         if hasattr(self, 'extra_headers'):
             for header in self.extra_headers:
                 res[header] = self.extra_headers[header]
         
         return res
-        
+
 class SerializeOrRedirect(DynamicResponse):
     """
     For normal requests, the user is redirected to the specified location.
     For API requests, the context is serialized and returned as JSON.
     """
-
+    
     def __init__(self, url, context={}, **kwargs):
-
+        
         super(SerializeOrRedirect, self).__init__(context, **kwargs)
         self.url = url
-
+    
     def render_response(self, request, response):
         
         if request.is_api:
             res = self.serialize()
         else:
             res = HttpResponseRedirect(self.url)
-
+        
         if hasattr(self, 'extra_headers'):
             for header in self.extra_headers:
                 res[header] = self.extra_headers[header]
-
+        
         return res
 
 class Serialize(DynamicResponse):
@@ -101,11 +132,11 @@ class Serialize(DynamicResponse):
     """
     
     def __init__(self, context={}, **kwargs):
-
+        
         super(Serialize, self).__init__(context, **kwargs)
-
+    
     def render_response(self, request, response):
-
+        
         res = self.serialize()
         
         if hasattr(self, 'extra_headers'):
